from typing import Callable, Dict

import pytest

from petisco.application.config.config import Config


@pytest.mark.end2end
def test_should_load_complete_petisco_yml(petisco_yml_path_end2end):

    filename = f"{petisco_yml_path_end2end}/petisco.all.yml"

    config = Config.from_filename(filename).unwrap()

    assert len(config.config_tasks.tasks.keys()) == 3

    assert isinstance(config.config_persistence.configs, Dict)
    assert isinstance(config.config_persistence.configs["petisco"].config, Callable)
    assert config.config_persistence.configs["petisco"].models is not None

    assert isinstance(config.config_providers.services_provider, Callable)
    assert isinstance(config.config_providers.repositories_provider, Callable)
    assert isinstance(config.config_notifier.provider, Callable)


@pytest.mark.end2end
def test_should_load_a_petisco_yml_with_no_events(petisco_yml_path_end2end):

    filename = f"{petisco_yml_path_end2end}/petisco.noevents.yml"

    config = Config.from_filename(filename).unwrap()

    assert isinstance(config.config_persistence.configs, Dict)
    assert isinstance(config.config_persistence.configs["petisco"].config, Callable)
    assert config.config_persistence.configs["petisco"].models is not None

    assert isinstance(config.config_providers.services_provider, Callable)
    assert isinstance(config.config_providers.repositories_provider, Callable)


@pytest.mark.end2end
def test_should_load_a_petisco_yml_with_events_no_subscribers(petisco_yml_path_end2end):

    filename = f"{petisco_yml_path_end2end}/petisco.events.nosubscribers.yml"

    config = Config.from_filename(filename).unwrap()

    assert isinstance(config.config_persistence.configs, Dict)
    assert isinstance(config.config_persistence.configs["petisco"].config, Callable)
    assert config.config_persistence.configs["petisco"].models is not None

    assert isinstance(config.config_providers.services_provider, Callable)
    assert isinstance(config.config_providers.repositories_provider, Callable)



from petisco import RabbitMqConnector
from petisco.event.configurer.infrastructure.rabbitmq_event_configurer import (
    RabbitMqEventConfigurer,
)
from petisco.event.queue.domain.queue_config import QueueConfig
from tests.modules.event.mothers.defaults import DEFAULT_ORGANIZATION, DEFAULT_SERVICE


class RabbitMqEventConfigurerMother:
    @staticmethod
    def default(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE)

    @staticmethod
    def with_retry_ttl_10ms(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector,
            DEFAULT_ORGANIZATION,
            DEFAULT_SERVICE,
            QueueConfig.default(default_retry_ttl=10),
        )

    @staticmethod
    def with_main_and_retry_ttl_10ms(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector,
            DEFAULT_ORGANIZATION,
            DEFAULT_SERVICE,
            QueueConfig.default(default_retry_ttl=10, default_main_ttl=10),
        )

    @staticmethod
    def with_main_and_retry_ttl_100ms(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector,
            DEFAULT_ORGANIZATION,
            DEFAULT_SERVICE,
            QueueConfig.default(default_retry_ttl=100, default_main_ttl=100),
        )

    @staticmethod
    def with_service(service: str, connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector,
            DEFAULT_ORGANIZATION,
            service,
            QueueConfig.default(default_retry_ttl=10),
        )

    @staticmethod
    def with_ttl_1s(connector: RabbitMqConnector = None):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector,
            DEFAULT_ORGANIZATION,
            DEFAULT_SERVICE,
            QueueConfig.default(default_retry_ttl=1000),
        )

    @staticmethod
    def with_queue_config(
        queue_config: QueueConfig, connector: RabbitMqConnector = None
    ):
        connector = RabbitMqConnector() if not connector else connector
        return RabbitMqEventConfigurer(
            connector, DEFAULT_ORGANIZATION, DEFAULT_SERVICE, queue_config
        )



import pytest
from meiga import Success
from meiga.assertions import assert_success

from petisco.tools.timer import timer


@pytest.mark.unit
def test_should_add_elapsed_time_to_results_when_use_timer_decorator():
    @timer
    def controller():
        return Success(({"message": "Ok"}, 200))

    result, elapsed_time = controller()

    assert_success(result)
    assert isinstance(elapsed_time, float)



from time import sleep

import pytest
from meiga import isFailure, Result, Error, isSuccess, Failure

from petisco import Event, event_handler, DEBUG
from petisco.domain.errors.critical_error import CriticalError
from petisco.event.shared.domain.event_subscriber import EventSubscriber

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.rabbitmq_event_bus_mother import RabbitMqEventBusMother
from tests.modules.event.mothers.rabbitmq_event_configurer_mother import (
    RabbitMqEventConfigurerMother,
)
from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents
from tests.modules.unit.mocks.fake_logger import FakeLogger
from tests.modules.unit.mocks.fake_notifier import FakeNotifier
from tests.modules.unit.mocks.log_message_mother import LogMessageMother


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_with_event_handler_when_success_consumer():
    spy = SpyEvents()
    logger = FakeLogger()

    @event_handler(logger=logger)
    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.without_retry()
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)

    first_logging_message = logger.get_logging_messages()[0]
    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_event_handler(
            operation="assert_consumer",
            message={"event": event.event_name, "body": event.to_json()},
        ).to_dict(),
    )


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_with_event_handler_with_default_parameters_when_success_consumer(
    given_any_petisco
):
    spy = SpyEvents()

    @event_handler()
    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isSuccess

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.without_retry()
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_with_event_handler_when_fail_consumer_without_retry():
    spy = SpyEvents()
    logger = FakeLogger()

    @event_handler(logger=logger)
    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isFailure

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.without_retry()
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)

    first_logging_message = logger.get_logging_messages()[0]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_event_handler(
            operation="assert_consumer",
            message={"event": event.event_name, "body": event.to_json()},
        ).to_dict(),
    )


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_with_event_handler_when_fail_consumer():
    spy = SpyEvents()
    logger = FakeLogger()

    @event_handler(logger=logger)
    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return isFailure

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.with_max_retries(1)
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)

    first_logging_message = logger.get_logging_messages()[0]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_event_handler(
            operation="assert_consumer",
            message={"event": event.event_name, "body": event.to_json()},
        ).to_dict(),
    )


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_with_event_handler_return_nothing():
    spy = SpyEvents()
    logger = FakeLogger()

    @event_handler(logger=logger)
    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.with_retry_ttl_10ms()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.with_max_retries(5)
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)  # If returns isFailure it would retry
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)

    first_logging_message = logger.get_logging_messages()[0]

    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_event_handler(
            operation="assert_consumer",
            message={"event": event.event_name, "body": event.to_json()},
        ).to_dict(),
    )


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_with_event_handler_notify_when_fail_consumer_with_critical_error(
    given_any_petisco
):
    spy = SpyEvents()
    logger = FakeLogger()
    notifier = FakeNotifier()

    class MyCriticalError(CriticalError):
        pass

    @event_handler(logger=logger, notifier=notifier)
    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        return Failure(MyCriticalError(Exception()))

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.without_retry()
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)

    first_logging_message = logger.get_logging_messages()[0]
    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_event_handler(
            operation="assert_consumer",
            message={"event": event.event_name, "body": event.to_json()},
        ).to_dict(),
    )
    assert notifier.publish_called
    assert notifier.publish_times_called == 1


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_publish_consume_with_event_handler_not_notify_when_fail_consumer_with_critical_error():
    spy = SpyEvents()
    logger = FakeLogger()
    notifier = FakeNotifier()

    @event_handler(logger=logger, notifier=notifier)
    def assert_consumer(event: Event) -> Result[bool, Error]:
        spy.append(event)
        raise Exception()

    event = EventUserCreatedMother.random()
    subscribers = [
        EventSubscriber(
            event_name=event.event_name,
            event_version=event.event_version,
            handlers=[assert_consumer],
        )
    ]

    configurer = RabbitMqEventConfigurerMother.default()
    configurer.configure_subscribers(subscribers)

    bus = RabbitMqEventBusMother.default()
    bus.publish(event)

    consumer = RabbitMqEventConsumerMother.without_retry()
    consumer.add_subscribers(subscribers)
    consumer.start()

    sleep(1.0)

    consumer.stop()
    configurer.clear()

    spy.assert_number_unique_events(1)
    spy.assert_first_event(event)
    spy.assert_count_by_event_id(event.event_id, 1)

    first_logging_message = logger.get_logging_messages()[0]
    assert first_logging_message == (
        DEBUG,
        LogMessageMother.get_event_handler(
            operation="assert_consumer",
            message={"event": event.event_name, "body": event.to_json()},
        ).to_dict(),
    )
    assert not notifier.publish_called



from typing import Dict

from petisco.event.bus.domain.interface_event_bus import IEventBus
from petisco.event.shared.domain.event import Event


class NotImplementedEventBus(IEventBus):
    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def publish(self, event: Event):
        pass

    def retry_publish_only_on_store_queue(self, event: Event):
        pass

    def stop(self):
        pass



from typing import Dict

from petisco import IAppService
from tests.end2end.flask_app.sum.infrastructure.sum_executor import SumExecutor


def services_provider() -> Dict[str, IAppService]:
    return {"sum": SumExecutor()}



def flask_headers_provider() -> dict:
    try:
        import connexion

        headers = connexion.request.headers
        headers = {h: v for h, v in headers.items() if isinstance(h, str)}
    except (RuntimeError, ImportError):
        headers = {}

    return headers



from typing import Callable

from time import sleep

import pytest
import os

from petisco.event.legacy.publisher.infrastructure.rabbitmq_event_publisher import (
    RabbitMQEventPublisher,
)
from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_connector import (
    RabbitMqConnector,
)
from petisco.event.legacy.subscriber.domain.config_event_subscriber import (
    ConfigEventSubscriber,
)

from petisco.event.shared.infrastructure.rabbitmq.rabbitmq_is_running_locally import (
    rabbitmq_is_running_locally,
)
from petisco.event.legacy.subscriber.domain.subscriber_handler import subscriber_handler
from petisco.event.legacy.subscriber.infrastructure.rabbitmq_event_subscriber import (
    RabbitMQEventSubscriber,
)

from petisco.event.shared.domain.event import Event
from meiga import isSuccess


def await_for_it(seconds: float = 1.5):
    sleep(seconds)


@pytest.fixture()
def given_any_config_event_subscriber(
    given_random_organization, given_random_service, given_random_topic
):
    def _given_any_config_event_subscriber(
        handler: Callable = lambda ch, method, properties, body: None,
    ):
        return ConfigEventSubscriber(
            organization=given_random_organization,
            service=given_random_service,
            topic=given_random_topic,
            handler=handler,
        )

    return _given_any_config_event_subscriber


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_create_a_rabbitmq_event_subscriber_and_then_close_it(
    given_any_config_event_subscriber
):
    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={"auth": given_any_config_event_subscriber()},
    )
    subscriber.start()

    await_for_it()

    info = subscriber.info()

    assert info == {
        "name": "RabbitMQEventSubscriber",
        "connection.is_open": True,
        "subscribers_status": {"auth": "subscribed"},
    }

    subscriber.stop()


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_create_a_rabbitmq_event_subscriber_and_try_to_delete_twice(
    given_any_config_event_subscriber
):
    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={"auth": given_any_config_event_subscriber()},
    )
    subscriber.start()
    await_for_it()
    subscriber.stop()
    subscriber.stop()


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_create_a_rabbitmq_event_subscriber_and_then_unsubscribe_all_when_not_subscribe_all_before(
    given_any_config_event_subscriber
):
    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={"auth": given_any_config_event_subscriber()},
    )
    subscriber.stop()


@pytest.mark.integration
def test_should_fail_subscriber_when_connection_parameter_are_not_valid(
    given_any_config_event_subscriber
):
    with pytest.raises(TypeError):
        _ = RabbitMQEventSubscriber(
            connector=None, subscribers={"auth": given_any_config_event_subscriber()}
        )


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_create_a_rabbitmq_event_subscriber_and_check_info(
    given_any_config_event_subscriber
):
    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={"auth": given_any_config_event_subscriber()},
    )
    info = subscriber.info()

    assert info == {
        "name": "RabbitMQEventSubscriber",
        "connection.is_open": False,
        "subscribers_status": {},
    }
    subscriber.stop()


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_throw_a_exception_if_try_to_start_a_subscriber_with_no_subscribers():
    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(), subscribers=None
    )

    with pytest.raises(RuntimeError):
        subscriber.start()


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_create_two_rabbitmq_event_subscriber_and_then_close_one_without_altering_the_other(
    given_any_config_event_subscriber
):
    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={"auth": given_any_config_event_subscriber()},
        connection_name="subscriber",
    )
    dl_subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={"dl-auth": given_any_config_event_subscriber()},
        connection_name="dl-subscriber",
    )

    subscriber.start()
    dl_subscriber.start()

    await_for_it()

    assert subscriber.info() == {
        "name": "RabbitMQEventSubscriber",
        "connection.is_open": True,
        "subscribers_status": {"auth": "subscribed"},
    }

    assert dl_subscriber.info() == {
        "name": "RabbitMQEventSubscriber",
        "connection.is_open": True,
        "subscribers_status": {"dl-auth": "subscribed"},
    }

    dl_subscriber.stop()

    await_for_it(4.0)

    assert subscriber.info() == {
        "name": "RabbitMQEventSubscriber",
        "connection.is_open": True,
        "subscribers_status": {"auth": "subscribed"},
    }

    assert dl_subscriber.info() == {
        "name": "RabbitMQEventSubscriber",
        "connection.is_open": True,
        "subscribers_status": {"dl-auth": "unsubscribed"},
    }
    subscriber.stop()


@pytest.mark.integration
@pytest.mark.skipif(
    not rabbitmq_is_running_locally(), reason="RabbitMQ is not running locally"
)
def test_should_work_successfully_a_happy_path_pub_sub_with_two_subscribers_and_closing_one_of_them(
    given_any_petisco,
    make_user_created_event,
    given_random_organization,
    given_random_service,
    given_random_topic,
):
    event_before_delete_letter_stop = make_user_created_event("user_id_1")
    event_after_delete_letter_stop_1 = make_user_created_event(
        "user_id_after_delete_letter_stop_1"
    )
    event_after_delete_letter_stop_2 = make_user_created_event(
        "user_id_after_delete_letter_stop_2"
    )
    event_after_delete_letter_stop_3 = make_user_created_event(
        "user_id_after_delete_letter_stop_3"
    )

    filename_main_handler = "filename_main_handler.txt"
    filename_main_handler_requeue = "filename_main_handler_requeue.txt"

    if os.path.exists(filename_main_handler):
        os.remove(filename_main_handler)

    if os.path.exists(filename_main_handler_requeue):
        os.remove(filename_main_handler_requeue)

    @subscriber_handler()
    def main_handler(event: Event):
        print(f"main_handler: {event.to_json()}")
        with open(filename_main_handler, "a+") as fm:
            fm.write(event.to_json() + "\n")
        return isSuccess

    @subscriber_handler()
    def main_handler_requeue(event: Event):
        print(f"main_handler_requeue: {event.to_json()}")
        with open(filename_main_handler_requeue, "a+") as fp:
            fp.write(event.to_json() + "\n")
        return isSuccess

    subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={
            "auth": ConfigEventSubscriber(
                organization=given_random_organization,
                service=given_random_service,
                topic=given_random_topic,
                handler=main_handler,
            )
        },
        connection_name="subscriber",
    )
    dl_subscriber = RabbitMQEventSubscriber(
        connector=RabbitMqConnector(),
        subscribers={
            "dl-auth": ConfigEventSubscriber(
                organization=given_random_organization,
                service=given_random_service,
                topic=given_random_topic,
                handler=main_handler_requeue,
                dead_letter=True,
            )
        },
        connection_name="dl-subscriber",
    )

    publisher = RabbitMQEventPublisher(
        connector=RabbitMqConnector(),
        organization=given_random_organization,
        service=given_random_service,
        topic=given_random_topic,
    )

    subscriber.start()
    dl_subscriber.start()

    await_for_it(1.5)

    publisher.publish(event_before_delete_letter_stop)

    await_for_it(1.5)

    dl_subscriber.stop()

    await_for_it(5.5)

    publisher.publish_events(
        [
            event_after_delete_letter_stop_1,
            event_after_delete_letter_stop_2,
            event_after_delete_letter_stop_3,
        ]
    )

    await_for_it(1.5)

    with open(filename_main_handler, "r") as fmm:
        lines = fmm.readlines()
        events = []
        for line in lines:
            events.append(Event.from_json(line))

        assert event_before_delete_letter_stop == events[0]
        assert event_after_delete_letter_stop_1 == events[1]
        assert event_after_delete_letter_stop_2 == events[2]
        assert event_after_delete_letter_stop_3 == events[3]

    assert not os.path.exists(filename_main_handler_requeue)
    os.remove(filename_main_handler)

    subscriber.stop()



import json


class WebhookRequestResult:
    @staticmethod
    def from_dict(kdict: dict):
        return WebhookRequestResult(
            headers=json.loads(kdict.get("headers")) if kdict.get("headers") else None,
            body=json.loads(kdict.get("body")) if kdict.get("body") else None,
        )

    def to_dict(self):
        return {
            "headers": json.dumps(self.headers) if self.headers else None,
            "body": json.dumps(self.body) if self.body else None,
        }

    def __init__(self, headers: dict, body: dict):
        self._set_headers(headers)
        self._set_body(body)
        super().__init__()

    def _set_headers(self, headers):
        try:
            self.headers = dict(headers)
        except Exception:
            self.headers = {}

    def _set_body(self, body):
        try:
            self.body = dict(body)
        except Exception:
            self.body = {}

    def __repr__(self):
        return json.dumps(self.to_dict())

    def __eq__(self, other):
        if issubclass(other.__class__, self.__class__) or issubclass(
            self.__class__, other.__class__
        ):
            return self.to_dict() == other.to_dict()
        else:
            return False



import pytest
from meiga import Success
from meiga.assertions import assert_failure
from meiga.decorators import meiga

from petisco.domain.value_objects.name import Name
from petisco.domain.errors.given_name_is_not_valid_error import GivenNameIsNotValidError
from petisco.domain.errors.length_limit_string_value_object_error import (
    ExceedLengthLimitValueObjectError,
)


@pytest.mark.unit
def test_should_declare_a_valid_name():

    value = "Rosalia"
    name = Name(value)

    assert isinstance(name, Name)
    assert name.value == value


@pytest.mark.unit
def test_should_declare_a_name_that_exceeds_default_length_limits():

    with pytest.raises(ExceedLengthLimitValueObjectError):
        Name(
            "Rosalia de Castro. Adios rios adios fontes, adios, regatos pequenos, adios, vista dos meus ollos, non sei cando nos veremos."
        )


@pytest.mark.unit
def test_should_declare_a_name_with_js_injection():

    with pytest.raises(GivenNameIsNotValidError):
        Name("<script>evil()</script>")


@pytest.mark.unit
def test_should_declare_a_name_with_empty_string():
    value = ""
    name = Name(value)

    assert isinstance(name, Name)
    assert name.value == value


@pytest.mark.unit
def test_should_raise_given_name_is_not_valid_when_value_is_not_a_string():

    with pytest.raises(GivenNameIsNotValidError):
        Name([0, 0])


@pytest.mark.unit
def test_should_raise_given_name_is_not_valid_when_value_is_a_string_with_4_byte_utf_chars():

    with pytest.raises(GivenNameIsNotValidError):
        Name("𝘼𝙡𝙚𝙭")


@pytest.mark.unit
@pytest.mark.parametrize(
    "input_name",
    [
        "毛泽东",
        "Milošević",
        "Müller",
        "Conceição",
        "Björk Guðmundsdóttir",
        "María-Jose Carreño Quiñones",
        "Борис Николаевич Ельцин",
        "John Q. Public",
        "John F.",
        "Nguyễn Tấn Dũng",
        "Øåąćłńśź",
        "öêãàõâôñ",
        "แมว",  # not working with thai marks ม้
        "Tōkairin",
    ],
)
def test_should_declare_a_valid_name_parametrizable(input_name):
    name = Name(input_name)

    assert isinstance(name.value, str)
    assert name.value == input_name


@pytest.mark.unit
def test_should_fail_when_declare_a_large_name_on_a_meiga_decorated_method():
    @meiga
    def controller():
        name = Name(
            "Rosalia de Castro. Adios rios adios fontes, adios, regatos pequenos, adios, vista dos meus ollos, non sei cando nos veremos."
        )
        return Success(name)

    result = controller()
    assert_failure(result, value_is_instance_of=ExceedLengthLimitValueObjectError)


@pytest.mark.unit
def test_should_fail_when_declare_a_no_valid_name_on_a_meiga_decorated_method():
    @meiga
    def controller():
        name = Name("<script>evil()</script>")
        return Success(name)

    result = controller()
    assert_failure(result, value_is_instance_of=GivenNameIsNotValidError)



from abc import ABCMeta, abstractmethod
from typing import Any


class IApplication:
    __metaclass__ = ABCMeta

    @abstractmethod
    def start(self):
        raise NotImplementedError

    @abstractmethod
    def get_app(self) -> Any:
        raise NotImplementedError



from meiga import Result

from petisco.controller.errors.http_error import HttpError
from petisco.controller.errors.invalid_token_http_error import InvalidTokenHttpError
from petisco.security.token_decoder.invalid_token_error import InvalidTokenError


class KnownResultFailureHandler:
    def __init__(self, result: Result):
        self.domain_error = result.value

        self.http_error = HttpError()
        self.is_a_result_known_error = False
        domain_error = result.value

        if isinstance(domain_error, InvalidTokenError):
            self.http_error = InvalidTokenHttpError(suffix=domain_error.message)
            self.is_a_result_known_error = True



import pytest
from petisco.domain.value_objects.uuid import Uuid, InvalidUuidError


@pytest.mark.unit
def test_should_construct_generate():
    uuid = Uuid.generate()

    assert len(uuid.value) == 36


@pytest.mark.unit
def test_should_inherit_from_uuid_and_construct_from_generated_value():
    class MyId(Uuid):
        pass

    uuid_value = Uuid.generate().value

    my_id = MyId(uuid_value)

    assert isinstance(my_id, MyId)
    assert my_id.value == uuid_value


@pytest.mark.unit
def test_should_inherit_from_uuid_and_generate():
    class MyId(Uuid):
        pass

    my_id = MyId.generate()

    assert isinstance(my_id, MyId)
    assert len(my_id.value) == 36


@pytest.mark.unit
def test_should_inherit_from_uuid_and_construct_from_invalid_value():
    class MyId(Uuid):
        pass

    invalid_value = "acme"

    with pytest.raises(InvalidUuidError) as excinfo:
        MyId(invalid_value)

    assert "acme" in str(excinfo.value)



from abc import ABCMeta
from typing import Dict

from petisco.domain.value_objects.client_id import ClientId
import copy


class PatternBase:

    __metaclass__ = ABCMeta

    @classmethod
    def info(cls) -> Dict:
        return {"name": cls.__name__}

    def _set_client_id(self, client_id: ClientId):
        self.client_id = client_id

    def with_client_id(self, client_id: ClientId):
        repository = copy.copy(self)
        repository._set_client_id(client_id)
        return repository

    def get_client_id(self) -> ClientId:
        if not hasattr(self, "client_id"):
            name = self.info().get("name")
            raise AttributeError(
                f"{name} needs client_id. Please, {name}.use with_client_id() to get a valid"
            )
        return self.client_id

    def get_client_id_value(self) -> str:
        client_id = self.get_client_id()
        return client_id.value



from typing import Dict

from petisco.event.legacy.publisher.domain.interface_event_publisher import (
    IEventPublisher,
)
from petisco.event.legacy.publisher.infrastructure.not_implemented_event_publisher import (
    NotImplementedEventPublisher,
)
from petisco.event.legacy.subscriber.domain.config_event_subscriber import (
    ConfigEventSubscriber,
)
from petisco.event.legacy.subscriber.domain.interface_event_subscriber import (
    IEventSubscriber,
)
from petisco.event.legacy.subscriber.infrastructure.not_implemented_event_subscriber import (
    NotImplementedEventSubscriber,
)


def event_publisher_provider() -> IEventPublisher:
    return NotImplementedEventPublisher()


def event_subscriber_provider(
    subscribers: Dict[str, ConfigEventSubscriber]
) -> IEventSubscriber:
    return NotImplementedEventSubscriber(subscribers)



from time import sleep

import pytest

from petisco.domain.value_objects.user_id import UserId
from petisco.frameworks.flask.flask_extension_is_installed import (
    flask_extension_is_installed,
)
from petisco.persistence.sqlalchemy.sqlalchemy_extension_is_installed import (
    sqlalchemy_extension_is_installed,
)


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
@pytest.mark.skipif(
    not sqlalchemy_extension_is_installed(),
    reason="SQLAlchemy extension is not installed",
)
def test_should_return_200_when_call_a_entry_point_with_required_jwt_type_token(
    petisco_client_flask_app, given_auth_token_headers_creator, given_any_name
):

    headers = given_auth_token_headers_creator(type_token="ADMIN_TOKEN")

    response = petisco_client_flask_app.post(
        "/petisco/user", headers=headers, data=dict(name=given_any_name.value)
    )

    sleep(1.0)

    assert response.status_code == 200


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
@pytest.mark.skipif(
    not sqlalchemy_extension_is_installed(),
    reason="SQLAlchemy extension is not installed",
)
def test_should_return_401_when_call_a_entry_point_with_required_jwt_type_token(
    petisco_client_flask_app, given_auth_token_headers_creator, given_any_name
):

    headers = given_auth_token_headers_creator(type_token="INVALID_TOKEN")

    response = petisco_client_flask_app.post(
        "/petisco/user", headers=headers, data=dict(name=given_any_name)
    )
    assert response.status_code == 401
    assert response.json["error"]["type"] == "InvalidTokenHttpError"


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
@pytest.mark.skipif(
    not sqlalchemy_extension_is_installed(),
    reason="SQLAlchemy extension is not installed",
)
def test_should_return_200_when_call_a_entry_point_with_required_jwt_type_token_and_user_id(
    petisco_client_flask_app, given_auth_token_headers_creator, given_any_name
):
    headers = given_auth_token_headers_creator(type_token="ADMIN_TOKEN")

    response = petisco_client_flask_app.post(
        "/petisco/user", headers=headers, data=dict(name=given_any_name.value)
    )

    sleep(1.0)

    assert response.status_code == 200
    user_id = response.json["user_id"]

    headers_with_user = given_auth_token_headers_creator(
        type_token="USER_TOKEN", user_id=UserId(user_id)
    )

    response = petisco_client_flask_app.get(
        "/petisco/user/name", headers=headers_with_user
    )
    assert response.json["name"] == given_any_name.value


@pytest.mark.end2end
@pytest.mark.skipif(
    not flask_extension_is_installed(), reason="Flask extension is not installed"
)
@pytest.mark.skipif(
    not sqlalchemy_extension_is_installed(),
    reason="SQLAlchemy extension is not installed",
)
def test_should_return_401_when_call_a_entry_point_with_required_jwt_type_token_with_user_and_user_is_not_available(
    petisco_client_flask_app, given_auth_token_headers_creator
):

    headers = given_auth_token_headers_creator(type_token="USER_TOKEN")

    response = petisco_client_flask_app.get("/petisco/user/name", headers=headers)

    assert response.status_code == 401



from meiga import Result, Error, NotImplementedMethodError

from abc import ABCMeta, abstractmethod


class ITokenManager:

    __metaclass__ = ABCMeta

    @abstractmethod
    def execute(self, headers: dict) -> Result[str, Error]:
        raise NotImplementedMethodError



from time import sleep

import pytest
from meiga import isSuccess, isFailure, BoolResult

from petisco import Event, subscriber_handler

from petisco.fixtures.testing_decorators import testing_with_rabbitmq
from petisco.logger.not_implemented_logger import NotImplementedLogger
from tests.modules.event.legacy.mothers.rabbitmq_event_publisher_mother import (
    RabbitMqEventPublisherMother,
)
from tests.modules.event.legacy.mothers.rabbitmq_event_subscriber_mother import (
    RabbitMqEventSubscriberMother,
)
from tests.modules.event.legacy.mothers.rabbitmq_queue_naming_mother import (
    RabbitMqQueueNamingMother,
)
from tests.modules.event.mothers.event_user_created_mother import EventUserCreatedMother
from tests.modules.event.mothers.rabbitmq_declarer_mother import RabbitMqDeclarerMother

from tests.modules.event.mothers.rabbitmq_event_consumer_mother import (
    RabbitMqEventConsumerMother,
)
from tests.modules.event.spies.spy_events import SpyEvents


@pytest.mark.integration
@testing_with_rabbitmq
def test_should_configure_one_services_with_modern_event_management_an_other_with_legacy():
    spy_legacy = SpyEvents()
    spy_modern = SpyEvents()
    event = EventUserCreatedMother.random()

    def step_1_execute_legacy():
        # Define legacy Subscriber
        @subscriber_handler(logger=NotImplementedLogger())
        def main_handler(event: Event):
            spy_legacy.append(event)
            return isFailure

        subscriber = RabbitMqEventSubscriberMother.default(main_handler)
        subscriber.start()

        # Define legacy Publisher
        legacy_publisher = RabbitMqEventPublisherMother.default()
        legacy_publisher.publish(event)

        sleep(1.0)

        subscriber.stop()

    def step_2_execute_modern():
        # Define modern Consumer
        def modern_handler(event: Event) -> BoolResult:
            spy_modern.append(event)
            return isSuccess

        legacy_queue = RabbitMqQueueNamingMother.legacy_dead_letter_queue()
        consumer = RabbitMqEventConsumerMother.default()
        consumer.add_handler_on_queue(legacy_queue, modern_handler)
        consumer.start()

        sleep(1.0)

        consumer.stop()

    rabbitmq = RabbitMqDeclarerMother.default()
    rabbitmq.delete_queue(RabbitMqQueueNamingMother.legacy_dead_letter_queue())

    step_1_execute_legacy()
    step_2_execute_modern()

    spy_legacy.assert_number_unique_events(1)
    spy_legacy.assert_first_event(event)

    spy_modern.assert_number_unique_events(1)
    spy_modern.assert_first_event(event)
    spy_modern.assert_count_by_event_id(event.event_id, 1)

    rabbitmq.delete_queue(RabbitMqQueueNamingMother.legacy_main_queue())
    rabbitmq.delete_queue(RabbitMqQueueNamingMother.legacy_dead_letter_queue())



from typing import Callable
from petisco.persistence.interface_persistence_connector import IPersistenceConnector
from petisco.persistence.sqlalchemy.sqlalchemy_persistence import SqlAlchemyPersistence
from petisco.persistence.sqlalchemy.sqlalchemy_persistence_config import (
    SqlAlchemyPersistenceConfig,
)


class SqlAlchemyPersistenceConnector(IPersistenceConnector):
    def __init__(
        self,
        config: SqlAlchemyPersistenceConfig,
        name: str,
        import_database_models: Callable = None,
    ):
        self.config = config
        self.import_database_models = import_database_models
        self.name = name

    def get_connection(self):
        connection = None
        if self.config.server:
            if self.config.server == "sqlite":
                connection = "{}:///{}".format(self.config.server, self.config.database)
            elif self.config.server == "mysql":
                connection = "{}+{}://{}:{}@{}:{}/{}".format(
                    self.config.server,
                    self.config.driver,
                    self.config.user,
                    self.config.password,
                    self.config.host,
                    self.config.port,
                    self.config.database,
                )
        return connection

    def execute(self):
        from sqlalchemy import create_engine
        from sqlalchemy.ext.declarative import declarative_base
        from sqlalchemy.orm import sessionmaker
        from sqlalchemy_utils import database_exists, create_database

        connection = self.get_connection()

        base = declarative_base()

        persistence = SqlAlchemyPersistence()
        persistence.sources[self.name] = {"base": base}

        self.import_database_models()

        if not connection:
            raise ConnectionError(
                "Petisco SqlAlchemyPersistenceConnector is not configured. "
                "Please check SqlAlchemyPersistenceConfig and add required values"
            )

        if self.config.server == "sqlite":
            engine = create_engine(
                connection,
                json_serializer=lambda obj: obj,
                json_deserializer=lambda obj: obj,
            )
        else:
            engine = create_engine(
                connection,
                pool_pre_ping=True,
                json_serializer=lambda obj: obj,
                json_deserializer=lambda obj: obj,
            )

        if not database_exists(engine.url):
            create_database(engine.url)
            persistence.sources[self.name]["base"].metadata.create_all(engine)

        session = sessionmaker(bind=engine)
        persistence.sources[self.name]["session"] = session



import time


def recurring_task():
    now = time.strftime("%A, %d. %B %Y %I:%M:%S %p")
    print(f"recurring_task: {now}")


def scheduled_task():
    now = time.strftime("%A, %d. %B %Y %I:%M:%S %p")
    print(f"scheduled_task: {now}")


def instant_task():
    now = time.strftime("%A, %d. %B %Y %I:%M:%S %p")
    print(f"instant_task: {now}")



from typing import Dict

from meiga import Result, Error, isSuccess, Success

from tests.end2end.flask_app.users_count.domain.interface_users_count_repository import (
    IUsersCountRepository,
)


class InMemoryUsersCountRepository(IUsersCountRepository):
    def __init__(self):
        self.num_users = 0

    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def increase(self) -> Result[bool, Error]:
        self.num_users += 1
        return isSuccess

    def decrease(self) -> Result[bool, Error]:
        self.num_users -= 1
        return isSuccess

    def count(self) -> Result[int, Error]:
        return Success(self.num_users)



from typing import Any
from unittest.mock import Mock

import pytest
from pika.adapters.blocking_connection import BlockingChannel
from pika.spec import Basic

from petisco import RabbitMqEventChaos


@pytest.mark.integration
def test_should_configure_with_envvars(monkeypatch):
    monkeypatch.setenv("EVENT_CHAOS_PERCENTAGE_SIMULATE_NACK", "1.0")
    monkeypatch.setenv("EVENT_CHAOS_DELAY_BEFORE_EVENT_HANDLER_SECONDS", "1.0")
    monkeypatch.setenv("EVENT_CHAOS_PERCENTAGE_SIMULATE_FAILURES", "1.0")
    monkeypatch.setenv(
        "EVENT_CHAOS_PROTECTED_ROUTING_KEYS", "dead_letter.store,dl-legacy"
    )

    chaos = RabbitMqEventChaos()

    assert chaos.info() == {
        "EVENT_CHAOS_PERCENTAGE_SIMULATE_NACK": 1.0,
        "EVENT_CHAOS_DELAY_BEFORE_EVENT_HANDLER_SECONDS": 1.0,
        "EVENT_CHAOS_PERCENTAGE_SIMULATE_FAILURES": 1.0,
        "EVENT_CHAOS_PROTECTED_ROUTING_KEYS": ["dead_letter.store", "dl-legacy"],
    }


@pytest.mark.integration
def test_should_create_default_rabbitmq_event_chaos():
    chaos = RabbitMqEventChaos()

    assert chaos.nack_simulation(None, None) is False
    chaos.delay()
    assert chaos.failure_simulation(None) is False


@pytest.mark.integration
def test_should_create_default_rabbitmq_event_chaos_with_env_percentage_simulate_nack_1(
    monkeypatch
):
    monkeypatch.setenv("EVENT_CHAOS_PERCENTAGE_SIMULATE_NACK", "1.0")

    chaos = RabbitMqEventChaos()

    mock_channel = Mock(BlockingChannel)
    mock_method = Mock(Basic.Deliver)
    mock_method.delivery_tag = Mock(Any)
    mock_method.routing_key = Mock(Any)

    assert chaos.nack_simulation(mock_channel, mock_method) is True
    chaos.delay()
    assert chaos.failure_simulation(mock_method) is False

    mock_channel.basic_nack.assert_called_once()


@pytest.mark.integration
def test_should_create_default_rabbitmq_event_chaos_with_env_percentage_simulate_failures_1(
    monkeypatch
):
    monkeypatch.setenv("EVENT_CHAOS_PERCENTAGE_SIMULATE_FAILURES", "1.0")
    monkeypatch.setenv("EVENT_CHAOS_DELAY_BEFORE_EVENT_HANDLER_SECONDS", "0.2")

    chaos = RabbitMqEventChaos()
    assert chaos.nack_simulation(None, None) is False
    chaos.delay()
    assert chaos.failure_simulation(None) is True


@pytest.mark.integration
def test_should_create_rabbitmq_event_chaos_with_percentage_simulate_nack_1():
    chaos = RabbitMqEventChaos(percentage_simulate_nack=1.0)

    mock_channel = Mock(BlockingChannel)
    mock_method = Mock(Basic.Deliver)
    mock_method.delivery_tag = Mock(Any)
    mock_method.routing_key = Mock(Any)

    assert chaos.nack_simulation(mock_channel, mock_method) is True
    chaos.delay()
    assert chaos.failure_simulation(mock_method) is False

    mock_channel.basic_nack.assert_called_once()


@pytest.mark.integration
def test_should_create_rabbitmq_event_chaos_with_percentage_simulate_nack_but_with_protected_routing_key():
    chaos = RabbitMqEventChaos(
        percentage_simulate_nack=1.0, protected_routing_keys=["routing_key"]
    )

    mock_channel = Mock(BlockingChannel)
    mock_method = Mock(Basic.Deliver)
    mock_method.delivery_tag = Mock(Any)
    mock_method.routing_key = "routing_key"

    assert chaos.nack_simulation(mock_channel, mock_method) is False
    chaos.delay()
    assert chaos.failure_simulation(mock_method) is False


@pytest.mark.integration
def test_should_create_rabbitmq_event_chaos_with_percentage_simulate_failure_but_with_protected_routing_key():
    chaos = RabbitMqEventChaos(
        percentage_simulate_failures=1.0, protected_routing_keys=["routing_key"]
    )

    mock_channel = Mock(BlockingChannel)
    mock_method = Mock(Basic.Deliver)
    mock_method.routing_key = "routing_key"

    assert chaos.nack_simulation(mock_channel, mock_method) is False
    chaos.delay()
    assert chaos.failure_simulation(mock_method) is False


@pytest.mark.integration
def test_should_create_rabbitmq_event_chaos_with_percentage_simulate_failure_and_nck_but_with_protected_routing_key():
    chaos = RabbitMqEventChaos(
        percentage_simulate_nack=1.0,
        percentage_simulate_failures=1.0,
        protected_routing_keys=["routing_key"],
    )

    mock_channel = Mock(BlockingChannel)
    mock_method = Mock(Basic.Deliver)
    mock_method.delivery_tag = Mock(Any)
    mock_method.routing_key = "routing_key"

    assert chaos.nack_simulation(mock_channel, mock_method) is False
    chaos.delay()
    assert chaos.failure_simulation(mock_method) is False


@pytest.mark.integration
def test_should_create_rabbitmq_event_chaos_with_percentage_simulate_failures_1():
    chaos = RabbitMqEventChaos(percentage_simulate_failures=1.0)

    mock_method = Mock(Basic.Deliver)
    mock_method.routing_key = Mock(Any)

    assert chaos.nack_simulation(None, None) is False
    chaos.delay()
    assert chaos.failure_simulation(mock_method) is True



from abc import ABCMeta, abstractmethod

from meiga import Result, Error, NotImplementedMethodError

from petisco.application.interface_application_service import IAppService


class ISumExecutor(IAppService):
    __metaclass__ = ABCMeta

    @abstractmethod
    def execute(self, value1: int, value2: int) -> Result[int, Error]:
        raise NotImplementedMethodError
